@@ -25,7 +25,9 @@
 #include <glib/gi18n.h>
 #include <stdlib.h>
 #include <math.h>
+#include <cairo-svg.h>
 
+#include "ch-cleanup.h"
 #include "ch-point-obj.h"
 #include "ch-graph-widget.h"
 
@@ -76,6 +78,7 @@ enum
 	PROP_START_Y,
 	PROP_STOP_X,
 	PROP_STOP_Y,
+	PROP_LAST
 };
 
 /**
@@ -360,6 +363,9 @@ ch_get_axis_label (ChGraphWidgetType axis, gdouble value)
 				/*Translators: This is %2i minutes %02i seconds*/
 				text = g_strdup_printf (_("%2im%02i"), minutes, seconds);
 			}
+		} else if (value > 0.f && seconds < 2) {
+			/* TRANSLATORS: This is ms*/
+			text = g_strdup_printf (_("%.0fms"), value * 1000.f);
 		} else {
 			/*Translators: This is %2i seconds*/
 			text = g_strdup_printf (_("%2is"), seconds);
@@ -389,25 +395,23 @@ ch_get_axis_label (ChGraphWidgetType axis, gdouble value)
  * @graph: This class instance
  * @cr: Cairo drawing context
  *
- * Draw the 10x10 dotted grid onto the graph.
+ * Draw the 10x10 grid onto the graph.
  **/
 static void
 ch_graph_widget_draw_grid (ChGraphWidget *graph, cairo_t *cr)
 {
 	guint i;
 	gdouble b;
-	gdouble dotted[] = {1., 2.};
 	gdouble divwidth  = (gdouble)graph->priv->box_width / 10.0f;
 	gdouble divheight = (gdouble)graph->priv->box_height / 10.0f;
 
 	cairo_save (cr);
 
 	cairo_set_line_width (cr, 1);
-	cairo_set_dash (cr, dotted, 2, 0.0);
 
 	/* do vertical lines */
-	cairo_set_source_rgb (cr, 0.1, 0.1, 0.1);
-	for (i=1; i<10; i++) {
+	cairo_set_source_rgb (cr, 0.9f, 0.9f, 0.9f);
+	for (i = 1; i < 10; i++) {
 		b = graph->priv->box_x + ((gdouble) i * divwidth);
 		cairo_move_to (cr, (gint)b + 0.5f, graph->priv->box_y);
 		cairo_line_to (cr, (gint)b + 0.5f, graph->priv->box_y + graph->priv->box_height);
@@ -415,7 +419,7 @@ ch_graph_widget_draw_grid (ChGraphWidget *graph, cairo_t *cr)
 	}
 
 	/* do horizontal lines */
-	for (i=1; i<10; i++) {
+	for (i = 1; i < 10; i++) {
 		b = graph->priv->box_y + ((gdouble) i * divheight);
 		cairo_move_to (cr, graph->priv->box_x, (gint)b + 0.5f);
 		cairo_line_to (cr, graph->priv->box_x + graph->priv->box_width, (int)b + 0.5f);
@@ -450,7 +454,7 @@ ch_graph_widget_draw_labels (ChGraphWidget *graph, cairo_t *cr)
 	cairo_save (cr);
 
 	/* do x text */
-	cairo_set_source_rgb (cr, 0, 0, 0);
+	cairo_set_source_rgb (cr, 0.2f, 0.2f, 0.2f);
 	for (i = 0; i < 11; i++) {
 		b = graph->priv->box_x + ((gdouble) i * divwidth);
 		value = ((length_x / 10.0f) * (gdouble) i) + (gdouble) graph->priv->start_x;
@@ -600,12 +604,11 @@ ch_graph_widget_draw_dot (cairo_t *cr, gdouble x, gdouble y, guint32 color)
 static void
 ch_graph_widget_draw_line (ChGraphWidget *graph, cairo_t *cr)
 {
-	gdouble oldx, oldy;
-	gdouble newx, newy;
 	GPtrArray *data;
 	GPtrArray *array;
 	ChGraphWidgetPlot plot;
 	ChPointObj *point;
+	gdouble x, y;
 	guint i, j;
 
 	if (graph->priv->data_list->len == 0) {
@@ -617,7 +620,7 @@ ch_graph_widget_draw_line (ChGraphWidget *graph, cairo_t *cr)
 	array = graph->priv->data_list;
 
 	/* do each line */
-	for (j=0; j<array->len; j++) {
+	for (j = 0; j < array->len; j++) {
 		data = g_ptr_array_index (array, j);
 		if (data->len == 0)
 			continue;
@@ -625,68 +628,67 @@ ch_graph_widget_draw_line (ChGraphWidget *graph, cairo_t *cr)
 
 		/* get the very first point so we can work out the old */
 		point = (ChPointObj *) g_ptr_array_index (data, 0);
-		oldx = 0;
-		oldy = 0;
-		ch_graph_widget_get_pos_on_graph (graph, point->x, point->y, &oldx, &oldy);
-		if (plot == CH_GRAPH_WIDGET_PLOT_POINTS || plot == CH_GRAPH_WIDGET_PLOT_BOTH)
-			ch_graph_widget_draw_dot (cr, oldx, oldy, point->color);
+		x = 0;
+		y = 0;
+		ch_graph_widget_get_pos_on_graph (graph, point->x, point->y, &x, &y);
 
-		for (i=1; i < data->len; i++) {
-			point = (ChPointObj *) g_ptr_array_index (data, i);
-
-			ch_graph_widget_get_pos_on_graph (graph, point->x, point->y, &newx, &newy);
-
-			/* ignore white lines */
-			if (point->color == 0xffffff) {
-				oldx = newx;
-				oldy = newy;
-				continue;
+		/* plot points */
+		if (plot == CH_GRAPH_WIDGET_PLOT_POINTS || plot == CH_GRAPH_WIDGET_PLOT_BOTH) {
+			ch_graph_widget_draw_dot (cr, x, y, point->color);
+			for (i = 1; i < data->len; i++) {
+				point = (ChPointObj *) g_ptr_array_index (data, i);
+				ch_graph_widget_get_pos_on_graph (graph, point->x, point->y, &x, &y);
+				ch_graph_widget_draw_dot (cr, x, y, point->color);
 			}
+		}
 
-			/* draw line */
-			if (plot == CH_GRAPH_WIDGET_PLOT_LINE || plot == CH_GRAPH_WIDGET_PLOT_BOTH) {
-				cairo_move_to (cr, oldx, oldy);
-				cairo_line_to (cr, newx, newy);
-				cairo_set_line_width (cr, 1.5);
+		/* plot lines */
+		if (plot == CH_GRAPH_WIDGET_PLOT_LINE || plot == CH_GRAPH_WIDGET_PLOT_BOTH) {
+
+			guint32 old_color = 0xffffff;
+			cairo_set_line_width (cr, 1.5);
+
+			for (i = 1; i < data->len; i++) {
+				point = (ChPointObj *) g_ptr_array_index (data, i);
+
+				/* ignore anything out of range */
+				if (point->x < graph->priv->start_x ||
+				    point->x > graph->priv->stop_x) {
+					continue;
+				}
+
+				/* ignore white lines */
+				if (point->color == 0xffffff)
+					continue;
+
+				/* is graph color the same */
+				ch_graph_widget_get_pos_on_graph (graph,
+								  point->x,
+								  point->y,
+								  &x, &y);
+				if (point->color == old_color) {
+					cairo_line_to (cr, x, y);
+					continue;
+				}
+
+				/* finish previous line */
+				if (i != 1)
+					cairo_stroke (cr);
+
+				/* start new color line */
+				old_color = point->color;
+				cairo_move_to (cr, x, y);
 				ch_graph_widget_set_color (cr, point->color);
-				cairo_stroke (cr);
 			}
 
-			/* draw data dot */
-			if (plot == CH_GRAPH_WIDGET_PLOT_POINTS || plot == CH_GRAPH_WIDGET_PLOT_BOTH)
-				ch_graph_widget_draw_dot (cr, newx, newy, point->color);
-
-			/* save old */
-			oldx = newx;
-			oldy = newy;
+			/* finish current line */
+			cairo_stroke (cr);
 		}
 	}
 
 	cairo_restore (cr);
 }
 
-/**
- * ch_graph_widget_draw_bounding_box:
- * @cr: Cairo drawing context
- * @x: The X-coordinate for the top-left
- * @y: The Y-coordinate for the top-left
- * @width: The item width
- * @height: The item height
- **/
-static void
-ch_graph_widget_draw_bounding_box (cairo_t *cr, gint x, gint y, gint width, gint height)
-{
-	/* background */
-	cairo_rectangle (cr, x, y, width, height);
-	cairo_set_source_rgb (cr, 1, 1, 1);
-	cairo_fill (cr);
-	/* solid outline box */
-	cairo_rectangle (cr, x + 0.5f, y + 0.5f, width - 1, height - 1);
-	cairo_set_source_rgb (cr, 0.1, 0.1, 0.1);
-	cairo_set_line_width (cr, 1);
-	cairo_stroke (cr);
-}
-
 /**
  * ch_graph_widget_draw:
  * @graph: This class instance
@@ -716,11 +718,22 @@ ch_graph_widget_draw (GtkWidget *widget, cairo_t *cr)
 				 (3 + graph->priv->box_x);
 
 	/* graph background */
-	ch_graph_widget_draw_bounding_box (cr, graph->priv->box_x, graph->priv->box_y,
-				     graph->priv->box_width, graph->priv->box_height);
+	cairo_rectangle (cr, graph->priv->box_x, graph->priv->box_y,
+			 graph->priv->box_width, graph->priv->box_height);
+	cairo_set_source_rgb (cr, 1, 1, 1);
+	cairo_fill (cr);
+
+	/* grid */
 	if (graph->priv->use_grid)
 		ch_graph_widget_draw_grid (graph, cr);
 
+	/* solid outline box */
+	cairo_rectangle (cr, graph->priv->box_x + 0.5f, graph->priv->box_y + 0.5f,
+			 graph->priv->box_width - 1, graph->priv->box_height - 1);
+	cairo_set_source_rgb (cr, 0.6f, 0.6f, 0.6f);
+	cairo_set_line_width (cr, 1);
+	cairo_stroke (cr);
+
 	/* -3 is so we can keep the lines inside the box at both extremes */
 	data_x = graph->priv->stop_x - graph->priv->start_x;
 	data_y = graph->priv->stop_y - graph->priv->start_y;
@@ -734,6 +747,43 @@ ch_graph_widget_draw (GtkWidget *widget, cairo_t *cr)
 	return FALSE;
 }
 
+/**
+ * ch_graph_widget_export_to_svg_cb:
+ **/
+static cairo_status_t
+ch_graph_widget_export_to_svg_cb (void *user_data, const unsigned char *data, unsigned int length)
+{
+	GString *str = (GString *) user_data;
+	_cleanup_free_ gchar *tmp = NULL;
+
+	tmp = g_strndup ((const gchar *) data, length);
+	g_string_append (str, tmp);
+	return CAIRO_STATUS_SUCCESS;
+}
+
+/**
+ * ch_graph_widget_export_to_svg:
+ **/
+gchar *
+ch_graph_widget_export_to_svg (ChGraphWidget *graph, guint width, guint height)
+{
+	GString *str;
+	cairo_surface_t *surface;
+	cairo_t *ctx;
+
+	g_return_val_if_fail (CH_IS_GRAPH_WIDGET (graph), NULL);
+
+	/* write the SVG data to a string */
+	str = g_string_new ("");
+	surface = cairo_svg_surface_create_for_stream (ch_graph_widget_export_to_svg_cb,
+						       str, width, height);
+	ctx = cairo_create (surface);
+	ch_graph_widget_draw (GTK_WIDGET (graph), ctx);
+	cairo_surface_destroy (surface);
+	cairo_destroy (ctx);
+	return g_string_free (str, FALSE);
+}
+
 /**
  * ch_graph_widget_new:
  * Return value: A new ChGraphWidget object.
